// File:	E391aDetectorConstruction.hh
// Date:	16-Apr-2001 G.Macharashvili Juelich

#ifndef E391aDetectorConstruction_h
#define E391aDetectorConstruction_h 1

#include "E391aBBarrelSD.hh"
#include "E391aSBarrelSD.hh"
#include "E391aCalorimeterSD.hh"
#include "E391aCCo4.hh"
#include "E391aCCo5.hh"

#include "E391aCCo3SensitiveDetector.hh"
//#include "C3.hh"
#include "E391aCH-SD.hh"
#include "E391aCH-Digi.hh"
#include "E391aCalorimeterDigitizer.hh"
#include "E391aCalorimeterFilter.hh"
#include "DummyDetector.hh"
#include "DummyDigitizer.hh"

#include "EMShowerModel.hh"

#include "G4Material.hh"
#include "G4UserLimits.hh"
#include "G4ThreeVector.hh"
#include "G4VUserDetectorConstruction.hh"
#include "globals.hh"
#include "G4Box.hh"
#include "G4Cons.hh"
#include "G4Trd.hh"
#include "G4Trap.hh"
#include "G4Tubs.hh"
#include "G4Para.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4SDManager.hh"
#include "G4RunManager.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "G4RotationMatrix.hh"
#include "G4DigiManager.hh"

class E391aDetectorConstruction : public G4VUserDetectorConstruction {

//static	const	G4int	ECNX=27 ;
//static	const	G4int	ECNY=27 ;
static	const	G4int	SBNumOfModules=16 ;
static	const	G4int	BBNumOfModules=32 ;
static	const	G4int	CHNumOfModules=12 ;

public:
		E391aDetectorConstruction() ;
		~E391aDetectorConstruction() ;

G4VPhysicalVolume*	Construct() ;
E391aCalorimeterConfig*	CalorimeterConfig() { return EMConfig ; }
E391aCalorimeterFilter*	CalorimeterFilter() { return EMFilter ; }

G4VPhysicalVolume*	World() { return PhyWorld ; }
G4VPhysicalVolume*	ECModulePhysicalVolume(int ix,int iy) ;
G4LogicalVolume*	ECLogicalVolume() { return LogECModule ; }

private:

G4VPhysicalVolume*	E391aConstruct() ;     
void			E391aMaterials() ;
void			E391aDefaultGeometry() ;

protected: 

DummyDetector		*DD ;

G4Element		*H,*C,*N,*O,*Ar,*Mn,*Fe,*Cr,*Ni,*F,*I,*Cs,*Ce,*W,*Pb ;
G4Material		*Al,*Cu,*Iron,*Air,*CsI,*CeF3 ;
G4Material		*HVacuum, *LVacuum , *Scintillator, *Mylar, *StainlessSteel ;
G4Material		*Tungsten, *Lead, *SBMat, *BBMat, *CCMat ;

G4UserLimits		*UL ;
  
// Exp Hall
G4Box			*SolWorld ;
G4LogicalVolume		*LogWorld ;
G4VPhysicalVolume	*PhyWorld ;

// Beam Light
G4Cons			*SolBeam ;
G4LogicalVolume		*LogBeam ;
G4VPhysicalVolume	*PhyBeam ;

	// EM Calorimeter (CsI) (Modular) ...
G4Box				*SolECModule ;
G4LogicalVolume			*LogECModule ;
G4VPhysicalVolume		*PhyECModule[27*27] ;
E391aCalorimeterConfig		*EMConfig ;
E391aCalorimeterSD		*DetEC ;
E391aCalorimeterDigitizer	*DigitizerEC ;
E391aCalorimeterFilter		*EMFilter ;
	// EM calorimeter PM Housings ...
G4Tubs			*SolPM ;
G4LogicalVolume		*LogPM ;
G4VPhysicalVolume	*PhyPM[27*27] ;

	// Collar Calorimeter (Modular) ...
G4Box				*SolCC ;
G4LogicalVolume			*LogCC ;
G4RotationMatrix		*RMCC[4] ;
G4VPhysicalVolume		*PhyCC[4] ;

	// Fine Sructure of Collar Calorimeter
	G4Box			*SolW ;
	G4LogicalVolume		*LogW ;
	G4VPhysicalVolume	*PhyW[25] ;
	G4Box			*SolS ;
	G4LogicalVolume		*LogS ;
	G4VPhysicalVolume	*PhyS[26] ;
	// Collar Calorimeter internal Supporting Box
	G4Box			*SolX ;
	G4LogicalVolume		*LogX ;
	G4VPhysicalVolume	*PhyX[4] ;

// Small Veto Barrel (Modular)
G4Trap			*SolSBModule ;
G4LogicalVolume		*LogSBModule ;
G4RotationMatrix	*RM[16] ;
G4VPhysicalVolume	*PhySBModule[16] ;
E391aSBarrelSD		*DetSB ;

// Big Veto Barrel (Modular)
G4Trap			*SolBBModule ;
G4LogicalVolume		*LogBBModule ;
G4RotationMatrix	*RMBB[32] ;
G4VPhysicalVolume	*PhyBBModule[32] ;
E391aBBarrelSD		*DetBB ;

// Upstream Beam Veto 1
G4Tubs			*SolUB1 ;
G4LogicalVolume		*LogUB1 ;
G4VPhysicalVolume	*PhyUB1 ;

// Upstream Beam Veto 2
G4Tubs			*SolUB2 ;
G4LogicalVolume		*LogUB2 ;
G4VPhysicalVolume	*PhyUB2 ;

// Downstream Beam Veto 1 == Collar Calo 04
	G4LogicalVolume		*LogCCo4 ;
// Downstream Beam Veto 2 == Collar Calo 05
	G4LogicalVolume		*LogCCo5 ;

// Beam Catcher
G4Tubs			*SolBCatcher ;
G4LogicalVolume		*LogBCatcher ;
G4VPhysicalVolume	*PhyBCatcher ;

// Charged Veto (Modular)
G4Tubs			*SolCH ;
G4LogicalVolume		*LogCH ;
G4RotationMatrix	*RMCH[12] ;
G4VPhysicalVolume	*PhyCH[12] ;
E391aChargedVetoDetector	*DetCH ;
E391aCHDigitizer	*DigiCH ;

// Charged Veto Internal
G4Tubs			*SolCHo ;
G4LogicalVolume		*LogCHo ;
G4VPhysicalVolume	*PhyCHo ;
//E391aChargedVetoInsSD	*DetCHo ;

	} ;

#endif

// EOF
