// File:	EnergyDigitizer.hh
// Date:	27-Nov-2001 G.Macharashvili Juelich

#ifndef EnergyDigitizer_h
#define EnergyDigitizer_h 1

#include "globals.hh"
#include "CLHEP/Random/RandGaussQ.h"
#include "CLHEP/Random/RandPoissonQ.h"

class EnergyDigitizer {

protected:

static	const	G4bool		PSigmaON=true ; // ON/OFF fluctuations ...
static	const	G4bool		ESigmaON=true ; // ON/OFF fluctuations ...
static	const	G4int		MaxCount=4096-1 ; // LeCroy FERA 4300 (?) 
static	const	G4int		MinCount=1 ;
static	const	G4int		StdScale=1 ; // QDC counts for 1*MeV energy

public:

	EnergyDigitizer(G4double xScale=StdScale,G4double xNoise=1,G4double = 0) ;
	~EnergyDigitizer() ;
  
	void		SetMeV2Npe(G4double m2n) ;
	void		SetAB(G4double a,G4double b) ;

	G4double	EnergyRandomize(G4double e) ;
	G4double	PedestalRandomize() ;
	G4double	operator()(G4double q) ;

private:

	G4double	Scale ;   // QDC counts for 1 MeV energy
	G4double	Noise ;   // Pedestal fluctuation sigma (in QDC ch.)
	G4double	MeV2Npe ; // Num of Photoelectrons for 1 MeV
	G4double	Ea , Eb ; // sE/E=Ea/sqrt(E[GeV]) + Eb (not squared sum)

	} ;

#endif

// EOF
