
// File:		BitVector.cc
// Created:		13-Mar-2001 G.Macharashvili JINR Dubna
// E-mail:		gogi@nu.jinr.ru
// Subject:		Bit Set Class (One Dimensional Base Class)
// Source:		05-Aug-1997 G.Macharashvili

#include "BitVector.h"

	BitVector::BitVector(BitNumType bits) {
		base=(char *) new char [(bits+7)/8] ;
		if(base==0) {
			cout.form("\nBitVector: Init Error ...\n") ;
			exit(1) ;
			}
		maxbit=bits-1 ;
		Clear() ;
//		cout.form("BitVector: size  %d B\n",(bits+7)/8) ;
		}

	BitVector::~BitVector() { delete [] base ; }

BitNumType
	BitVector::MaxBit() const { return maxbit ; }

void	BitVector::Set(BitNumType bit1,BitNumType bit2) {
	for (BitNumType c=bit1 ; c<=bit2 ; c++) Set(c) ;
	}

void	BitVector::Set(BitNumType bit) {
	if (bit>=0 && bit<=maxbit) *(base+bit/8)|=(1<<(bit%8)) ;
	}

void	BitVector::Set() {
	for (int i=0 ; i<maxbit/8+1 ; i++ ) base[i]=0xFF ;
	}

void	BitVector::Clear(BitNumType bit1,BitNumType bit2) {
	for (BitNumType c=bit1 ; c<=bit2 ; c++) Clear(c) ;
	}

void	BitVector::Clear(BitNumType bit) {
	if (bit>=0 && bit<=maxbit) *(base+bit/8)&=~(1<<(bit%8)) ;
	}

void	BitVector::Clear() {
	for (int i=0 ; i<maxbit/8+1 ; i++ ) base[i]=0x0 ;
	}

void	BitVector::Invert(BitNumType bit1,BitNumType bit2) {
	for (BitNumType c=bit1 ; c<=bit2 ; c++) Invert(c) ;
	}

void	BitVector::Invert(BitNumType bit) {
	Test(bit)==1 ? Clear(bit) : Set(bit) ;
	}

void	BitVector::Invert() {
	for (int i=0 ; i<maxbit/8+1 ; i++ ) base[i]=~base[i] ;
	}

int	BitVector::Test(BitNumType bit1,BitNumType bit2) const {
	int ret=0 ;
	for (BitNumType c=bit1 ; c<=bit2 && ret==0 ; c++) ret|=Test(c) ;
	return ret ;
	}

int	BitVector::Test(BitNumType bit) const {
	if (bit<0 || bit>maxbit) return 0 ;
	return ((*(base+bit/8)>>(bit%8))&1) ;
	}

int	BitVector::Get(BitNumType bit) const {
	return ((*(base+bit/8)>>(bit%8))&1) ;
	}

void	BitVector::Put(BitNumType bit,int val) {
	val==0 ? Clear(bit) : Set(bit) ;
	}

BitNumType
	BitVector::FindFirstSet(BitNumType start_bit,BitNumType end_bit) const {
	unsigned int i=0 , j ;
	char *b=base ;
	unsigned char a ;
	while ( i<=end_bit ) {
		if ( ! (a = *b++) ) i+=8 ;
		else {
			j=8 ;
			while ( j-- ) {
				if ( a&1 && i>=start_bit ) return i ;
				a>>=1 ;
				i++ ;
				}
			}
		}
	return (ERR) ;
	}

BitNumType
	BitVector::FindFirstClear(BitNumType start_bit,BitNumType end_bit) const {
	unsigned int i=0 , j ;
	char *b=base ;
	unsigned char a ;
	while ( i<=end_bit ) {
		if ( (a = *b++) == 0xFF ) i+=8 ;
		else {
			j=8 ;
			while ( j-- ) {
				if (!(a&1) && i>=start_bit ) return i ;
				a>>=1 ;
				i++ ;
				}
			}
		}
	return (ERR) ;
	}

BitNumType	BitVector::SetBits() const {
	BitNumType counter=0 , current_bit=0 , end_bit=maxbit ;
	do {
		current_bit=FindFirstSet(current_bit,end_bit) ;
		if (current_bit==ERR) break ;
		current_bit++ ;
		counter++ ;
		} while (current_bit<=end_bit) ;
	return counter ;
	}

BitNumType	BitVector::ClearBits() const {
	BitNumType counter=0 , current_bit=0 , end_bit=maxbit ;
	do {
		current_bit=FindFirstClear(current_bit,end_bit) ;
		if (current_bit==ERR) break ;
		current_bit++ ;
		counter++ ;
		} while (current_bit<=end_bit) ;
	return counter ;
	}

void	BitVector::Print(BitNumType nb) {
	(nb==0 || nb>maxbit) ? Print(0,maxbit) : Print(0,nb-1) ;
	}

void	BitVector::Print(BitNumType nb1,BitNumType nb2) {
	if (nb1>nb2) {
		BitNumType tmp=nb1 ;
		nb1=nb2 ;
		nb2=tmp ;
		}
	if (nb2>maxbit) nb2=maxbit ;
	cout.form("\nBitVector::Print() (%u)  Bits:[%d..%d]\n",maxbit+1,nb1,nb2) ;
	int b2=nb2/8 ;
	int b1=nb1/8 ;
	int curr_byte=b1 ;
	for (curr_byte=b2 ; curr_byte>=b1 ; curr_byte--) {
		int bl=8 ;
		cout.form(" ") ;
			while(bl--){
			unsigned b=*(base+curr_byte)>>bl ;
			if (bl+curr_byte*8>nb2) cout.form(".") ;
			else if (bl+curr_byte*8<nb1) cout.form(".") ;
			else cout.form("%1d",(int)b&1) ;
			}
		}
	}

void	BitVector::Insert(long src,BitNumType bit1,BitNumType bit2) {
	if (bit1>bit2) {
		BitNumType tmp=bit1 ;
		bit1=bit2 ;
		bit2=tmp ;
		}
	if (bit2-bit1>24)  return ; // Safety Measure
	if (bit2+8>maxbit) return ; // Safety Measure
	unsigned long m=~(-1l<<(bit2-bit1+1)) ;
	unsigned long n=src&m ;
	*(unsigned long *)(base+bit1/8)&=~(m<<bit1%8) ;
	*(unsigned long *)(base+bit1/8)|=(n<<bit1%8) ;
	}

void	BitVector::Insert(const char* s,BitNumType bit1,BitNumType bit2) {
	char *w=(char *)s+strlen(s) ;
	BitNumType i=bit1 ;
	do{
		switch (*--w) {
			default:
			case '0':  Clear(i++) ;
				   break ;
			case '1':  Set(i++) ;
				   break ;
			case ' ':
			case '\t':
				   break ;
			}
		} while (!(i>bit2 || w==s)) ;
	}

unsigned long	BitVector::Extract(BitNumType bit1,BitNumType bit2) {
	if (bit1>bit2) {
		BitNumType tmp=bit1 ;
		bit1=bit2 ;
		bit2=tmp ;
		}
	unsigned long m=~(-1l<<(bit2-bit1+1)) ;
	return ((*(unsigned long *)(base+bit1/8)>>bit1%8)&m) ;
	}

// EOF
