// File:	DummyDigitizer.cc
// Date:	09-Oct-2001 G.Macharashvili Juelich

// CAUTION: Hits collection name and Digi collection name 
// should be the same.

#include "DummyDigitizer.hh"

DummyDigitizer::~DummyDigitizer() { delete [] DigiID ; }

DummyDigitizer::DummyDigitizer(G4String Name,int NumOfModules) : E391aVDigitizerModule(Name) {
	xMaxDigi=NumOfModules ;
	DigiID=new G4int[xMaxDigi] ;
	collectionName.insert(collectionName.begin(),Name) ; // G4 v.3 and later ...
	}

G4double DummyDigitizer::PedestalRand() { 
	return (PSigmaON) ? rint(RandGaussQ::shoot(0,QDCPSigma)) : 0 ;
	}

G4double DummyDigitizer::QDC(G4double e) {
	double	Q=(ESigmaON) ? rint(e*MeV2QDC+PedestalRand()) : e/MeV2QDC ;
	if (Q>0x8FFF) Q=0x8FFF ;
	return (Q>0) ? Q : 0 ;
	}

void	DummyDigitizer::MakeDigi(E391aHit *H) {
	G4double Q=QDC(H->EDep()) ;
	G4double T=TDC(H->Time()) ;
	if (Q<0.99) return ;
	G4int k=H->DetectorID()%10000 ;
//	if (k>=xMaxDigi) {
//		G4cout.form("DetectorID()=%d  ",H->DetectorID()) ;
//		G4cout.flush() ;
//		return ;
//		}
	if (DigiID[k]==-1) DigiID[k]=DC->insert(new E391aDigi(H->DetectorID(),Q,T))-1 ;
	else (*DC)[DigiID[k]]->AddQ(Q) ;
	}
			
void DummyDigitizer::Digitize() {
	int k ;
static	G4int	HiCID=-1 ;
	if (HiCID==-1) HiCID=G4DigiManager::GetDMpointer()->GetHitsCollectionID(moduleName) ;
	HC=(E391aHitsCollection*)G4DigiManager::GetDMpointer()->GetHitsCollection(HiCID) ;
	if (HC==0) return ;
	G4int	NumOfHits=HC->entries() ;
	if (NumOfHits==0) return ;
// Hit control print ...
//+	for (k=0 ; k<NumOfHits ; k++) (*HC)[k]->Print() ;
	Digitize(HC) ;
	}

void DummyDigitizer::Digitize(E391aHitsCollection* HC) {
//	G4cout<<moduleName<<"  Dummy ..." << endl ;
	G4int k , NumOfHits=HC->entries() ;
	for (k=0 ; k<xMaxDigi ; k++) DigiID[k]=-1 ;
	DC=new E391aDigiCollection(moduleName.data(),collectionName[0]) ;
	StoreDigiCollection(DC) ;
	for (k=0 ; k<NumOfHits ; k++) MakeDigi((*HC)[k]) ;
	}

// EOF
