// File:	E391aBBarrelSD.cc
// Date:	10-Apr-2001 G.Macharashvili Juelich

#include "E391aBBarrelSD.hh"

// Steps after first hit at current detector appears,
// At the Hit class initialization...
//
// 1. Get Ekin. What fun: DeltaE=Fun(Ekin) ? DeltaE=k*Ekin ?
// 2. Use parametrization.
// 3. Smearing signals (DeltaE and Time).
// 4. Digitization: MeV->Npe.
// 5. How to account the energy flow and leakige.
// 6. After all that kill track and secondaries.

E391aBBarrelSD::E391aBBarrelSD(
	G4String DetName,
	E391aDetectorConstruction* det,
	G4int	NumOfModules)
		:G4VSensitiveDetector(DetName),xDet(det) {
		xNumOfModules=NumOfModules ;
		collectionName.insert(DetName) ;
		HitID=new G4int[xNumOfModules] ;
		}

E391aBBarrelSD::~E391aBBarrelSD() { delete [] HitID ; }

void E391aBBarrelSD::Initialize(G4HCofThisEvent* HCE) {
//	G4cout<<"E391aBBarrelSD::Initialize() ...\n" ;
	HC=new E391aHitsCollection(SensitiveDetectorName,collectionName[0]) ;
	for(G4int j=0 ; j<xNumOfModules ; j++) HitID[j]=-1 ;
	}

G4bool E391aBBarrelSD::ProcessHits(G4Step* Stp,G4TouchableHistory* ROhist) {
//	G4cout<<"E391aBBarrelSD::ProcessHits() ... \n" ;
	G4int		module=Stp->GetPreStepPoint()->GetPhysicalVolume()->GetCopyNo() ;
	if (module<0 || module>xNumOfModules-1) return false ;
	G4int		id=xBaseIdent*10000+module ;
	G4double	gt=Stp->GetPreStepPoint()->GetGlobalTime() ;
	G4double	De=Stp->GetTotalEnergyDeposit() ;
	G4double	Sl=Stp->GetStepLength() ;
	G4int		Pid=-1 ;
	if (De<=0) return false ;
// History check/ where we are
	G4TouchableHistory* TH=(G4TouchableHistory*)Stp->GetPreStepPoint()->GetTouchable() ;
	G4String	VN=TH->GetVolume()->GetName() ;
// Mandatory (standard) logic
	if (HitID[module]==-1) {
		E391aHit* Hit=new E391aHit(id,Pid,0,De,gt) ;
		HitID[module]=HC->insert(Hit)-1 ;
		}
	else {
		(*HC)[HitID[module]]->AddEDep(De) ;
		}
// Kill Track ...
	Stp->GetTrack()->SetTrackStatus(fKillTrackAndSecondaries) ;

	return true ;
	}

void	E391aBBarrelSD::EndOfEvent(G4HCofThisEvent* HCE) {
//	G4cout<<"E391aBBarrelSD::EndOfEvent() ...\n" ;
static	G4int	HCollectionID=-1 ;
	if (HCollectionID<0) {
		HCollectionID=G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]) ;
		}
	HCE->AddHitsCollection(HCollectionID,HC) ;
//	for (int k=0 ; k<xNumOfModules ; k++) {
//		if (HitID[k]!=-1) (*HC)[HitID[k]]->Print() ;
//		}
	}

void E391aBBarrelSD::clear() {} 

void E391aBBarrelSD::DrawAll() {} 

void E391aBBarrelSD::PrintAll() {} 

// EOF
