// File:	E391aCCo3SensitiveDetector.cc
// Date:	25-Nov-2001 G.Macharashvili Dubna

#include "E391aCCo3SensitiveDetector.hh"

E391aCC3SensitiveDetector::E391aCC3SensitiveDetector(G4String DetName) : E391aVSensitiveDetector(DetName,4) {
	collectionName.insert(DetName) ;

	CC3NTrk=new TH1F("CC3NTrk","Secondary tracks number in CCo3",100,0,500) ;
		CC3NTrk->SetXTitle("Track number") ;
		CC3NTrk->SetYTitle("Number of events") ;
// ...
	CC3Spect=new TH1F("CC3Spect","CCo3 Hitted #gamma Energy",200,0,0.4) ;
		CC3Spect->SetXTitle("#gamma energy,  GeV") ;
		CC3Spect->SetYTitle("Number of events") ;

	CC3Dep=new TH1F("CC3Dep","Deposited Energy in CCo3",100,0,50) ;
		CC3Dep->SetXTitle("Deposited energy,  MeV") ;
		CC3Dep->SetYTitle("Number of events") ;
const	G4int		nn=350 ;
const	G4double	x1=-175*mm ;
const	G4double	y1=-175*mm ;
const	G4double	x2=+175*mm ;
const	G4double	y2=+175*mm ;
	CC3HitXY1=new TH2F("CC3HitXY1","Hit point distribution in CCo3 front",nn,x1,x2,nn,y1,y2) ;
		CC3HitXY1->SetXTitle("X coordinate,  mm") ;
		CC3HitXY1->SetYTitle("Y coordinate,  mm") ;
	CC3HitXY1u=new TH2F("CC3HitXY1u","Hit point distribution in CCo3 front.Und",nn,x1,x2,nn,y1,y2) ;
		CC3HitXY1u->SetXTitle("X coordinate,  mm") ;
		CC3HitXY1u->SetYTitle("Y coordinate,  mm") ;
	CC3HitXY2=new TH2F("CC3HitXY2","Hit point distribution in CCo3 In",nn,x1,x2,nn,y1,y2) ;
		CC3HitXY2->SetXTitle("X coordinate,  mm") ;
		CC3HitXY2->SetYTitle("Y coordinate,  mm") ;
	CC3HitXY2u=new TH2F("CC3HitXY2u","Hit point distribution in CCo3 In.Und",nn,x1,x2,nn,y1,y2) ;
		CC3HitXY2u->SetXTitle("X coordinate,  mm") ;
		CC3HitXY2u->SetYTitle("Y coordinate,  mm") ;
	CC3EkEvisS=new TH2F("CC3EkEvisS","Deposited energy in CCo3 Scintillator",100,0,1, 100,0,0.1) ;
		CC3EkEvisS->SetXTitle("Particle energy,  GeV") ;
		CC3EkEvisS->SetYTitle("Deposited energy,  GeV") ;
	CC3EkEvisW=new TH2F("CC3EkEvisW","Deposited energy in CCo3 Tungsten",100,0,1, 100,0,1) ;
		CC3EkEvisW->SetXTitle("Particle energy,  GeV") ;
		CC3EkEvisW->SetYTitle("Deposited energy,  GeV") ;
	CC3EkEvis=new TH2F("CC3EkEvis","Total deposited energy in CCo3",100,0,1, 100,0,1) ;
		CC3EkEvis->SetXTitle("Particle energy,  GeV") ;
		CC3EkEvis->SetYTitle("Deposited energy,  GeV") ;
	CC3EsEw=new TH2F("CC3EsEw","CCo3: #DeltaE_{s}/#DeltaE_{w} vs E_{kin}",100,0,1,100,0,0.4) ;
		CC3EsEw->SetXTitle("Particle energy,  GeV") ;
		CC3EsEw->SetYTitle("#DeltaE_{s}/#DeltaE_{w}") ;
	CC3Effo=new TH1F("CC3Effo","Initial Energy in CCo3",25,0,500) ;
		CC3Effo->SetXTitle("Energy,  MeV") ;
		CC3Effo->SetYTitle("Number of events") ;
	CC3Eff=new TH1F("CC3Eff","Detection inefficiency in CCo3 (#DeltaE<1 MeV)",25,0,500) ;
		CC3Eff->SetXTitle("Energy,  MeV") ;
		CC3Eff->SetYTitle("Number of events") ;
	CC3LocXY=new TH2F("CC3LocXY","CCo3: Local XY",120,-120,120,240,-60,60) ;
		CC3LocXY->SetXTitle("X local, mm") ;
		CC3LocXY->SetYTitle("Y local, mm") ;
	CC3EI=new TH1F("CC3EI","Exit/Enter/...",10,0,10) ;
		CC3EI->SetXTitle("EI+EI2*3") ;
		CC3EI->SetYTitle("Number of events") ;
		}

E391aCC3SensitiveDetector::~E391aCC3SensitiveDetector() {}

void E391aCC3SensitiveDetector::Initialize(G4HCofThisEvent* HCE) {
	HC=new E391aHitsCollection(SensitiveDetectorName,collectionName[0]) ;
	MaxTrack=0 ;
	InitHit=0 ;
	}

G4bool E391aCC3SensitiveDetector::ProcessHits(G4Step* Stp,G4TouchableHistory* RO) {
	G4int GType=StepClassifier(Stp) ;
//	Module=CopyNo() ; // not used ...

	CNo=0 ; 
	if (Stp->GetPreStepPoint()->GetPhysicalVolume()->GetName()=="CCo3.S") CNo=0 ;
	if (Stp->GetPreStepPoint()->GetPhysicalVolume()->GetName()=="CCo3.W") CNo=1 ;

	switch(GType) {
		case 0:
			return false ;
			break ; // errors
		case 1:
			if (!InitHit) {
				CreateHits() ;
				SetInitialParticleData(Stp) ;
				InitHit++ ;
				}
			else AddInitialParticleData(Stp) ;
			break ;
		case 2:
			if (Stp->GetTotalEnergyDeposit()>0) {
				if (!InitHit) {
					CreateHits() ;
					InitHit++ ;
					}
				(*HC)[CNo]->Add(Stp) ;
				}
			break ;
		}
	return true ;
	}

void	E391aCC3SensitiveDetector::CreateHits() {
	for (int k=0 ; k<xHitMax ; k++) {
		HC->insert(new E391aHit(CCBaseID*10000+k)) ;
		}
	}

void	E391aCC3SensitiveDetector::SetInitialParticleData(G4Step *S) {
	for (int k=0 ; k<xHitMax ; k++) {
		(*HC)[k]->SetKineticEnergy(S->GetPreStepPoint()->GetKineticEnergy()) ;
		(*HC)[k]->SetParticleID(ParID()) ;
		(*HC)[k]->SetPos(S->GetPreStepPoint()->GetPosition()) ;
		(*HC)[k]->SetDir(S->GetPreStepPoint()->GetMomentumDirection()) ;
		(*HC)[k]->SetEDep(S->GetTotalEnergyDeposit()) ;
		(*HC)[k]->SetTime(S->GetPreStepPoint()->GetGlobalTime()) ;
		}
	}

void	E391aCC3SensitiveDetector::AddInitialParticleData(G4Step *S) {
	for (int k=0 ; k<xHitMax ; k++) {
		(*HC)[k]->AddKineticEnergy(S->GetPreStepPoint()->GetKineticEnergy()) ;
		}
	}

void	E391aCC3SensitiveDetector::EndOfEvent(G4HCofThisEvent* HCE) {
//	G4cout<<"EndOfEvent with " ;
//	G4cout.form("%d Hits \n",HC->entries()) ;
//	G4cout.flush() ;

	if (HC->entries()==0) return ; /////////////////

// Hist Fill ...
	G4double Ek , DS , DW ;
	G4double xx , yy , zz ;
	G4bool	CC3Hit=false ;
	G4bool	CC3HitFront=true ;
	if ((*HC)[0]->ParticleID()!=-1) {
		CC3Hit=true ;
		xx=(*HC)[0]->X() ;
		yy=(*HC)[0]->Y() ;
		zz=(*HC)[0]->Z() ;
		DS=(*HC)[0]->EDep() ;
		DW=(*HC)[1]->EDep() ;
		Ek=(*HC)[0]->KineticEnergy() ;
		if ((*HC)[0]->Z()>5990*mm+1*mm) CC3HitFront=false ;
		if ((*HC)[0]->ParticleID()!=0) CC3Hit=false ;
		}
	if (CC3Hit) {
		CC3NTrk->Fill(float(MaxTrack)) ;
		CC3Spect->Fill(Ek/GeV) ;
		CC3Effo->Fill(Ek) ;
		CC3Dep->Fill(DS) ;
		CC3EkEvisS->Fill(Ek/GeV,DS/GeV) ;
		CC3EkEvisW->Fill(Ek/GeV,DW/GeV) ;
		CC3EkEvis->Fill(Ek/GeV,(DS+DW)/GeV) ;
		if (DW>0) CC3EsEw->Fill(Ek/GeV,DS/DW) ;
		if (DS > ESensitivityThreshold) {
			CC3Eff->Fill(Ek) ;
			}
		else	{
			if (CC3HitFront) CC3HitXY1u->Fill(xx,yy) ;
			else		 CC3HitXY2u->Fill(xx,yy) ;
			}
		if (CC3HitFront) CC3HitXY1->Fill(xx,yy) ;
		else		 CC3HitXY2->Fill(xx,yy) ;
		}

static	G4int	HCID=-1 ;
	if (HCID<0) HCID=G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]) ;
	HCE->AddHitsCollection(HCID,HC) ;
//G4cout<<endl ;
//G4cout.flush() ;

	}

// EOF
