// File:	E391aCCo4-SD.cc
// Date:	25-Sep-2001 G.Macharashvili Juelich

#include "E391aCCo4-SD.hh"

E391aCCo4SensitiveDetector::E391aCCo4SensitiveDetector(G4String DetName)
	:G4VSensitiveDetector(DetName) {
		collectionName.insert(DetName) ;
		HitID=new G4int[xHitMax] ;

	CCo4NTrk=new TH1F("CCo4NTrk","Secondary tracks number in CCo4",100,0,100) ;
		CCo4NTrk->SetXTitle("Track number") ;
		CCo4NTrk->SetYTitle("Number of events") ;
	CCo4Spect=new TH1F("CCo4Spect","Initial Energy in CCo4",200,0,0.4) ;
		CCo4Spect->SetXTitle("energy,  GeV") ;
		CCo4Spect->SetYTitle("Number of events") ;
	CCo4Dep=new TH1F("CCo4Dep","Deposited Energy in CCo4",100,0,50) ;
		CCo4Dep->SetXTitle("Deposited energy,  MeV") ;
		CCo4Dep->SetYTitle("Number of events") ;
	CCo4HitXY=new TH2F("CCo4HitXY","Hit point distribution in CCo4",100,-300,+300,100,-300,+300) ;
		CCo4HitXY->SetXTitle("X coordinate,  mm") ;
		CCo4HitXY->SetXTitle("Y coordinate,  mm") ;
	CCo4HitXYu=new TH2F("CCo4HitXYu","Hit point distribution in CCo4 Undetected",100,-300,+300,100,-300,+300) ;
		CCo4HitXYu->SetXTitle("X coordinate,  mm") ;
		CCo4HitXYu->SetXTitle("Y coordinate,  mm") ;
	CCo4Effo=new TH1F("CCo4Effo","Initial Energy in CCo4",25,0,500) ;
		CCo4Effo->SetXTitle("Energy,  MeV") ;
		CCo4Effo->SetYTitle("Number of events") ;
	CCo4Eff=new TH1F("CCo4Eff","Detection inefficiency in CCo4 (#DeltaE<1 MeV)",25,0,500) ;
		CCo4Eff->SetXTitle("Energy,  MeV") ;
		CCo4Eff->SetYTitle("Number of events") ;
		}

E391aCCo4SensitiveDetector::~E391aCCo4SensitiveDetector() { delete [] HitID ; }

void E391aCCo4SensitiveDetector::Initialize(G4HCofThisEvent* HCE) {
//oo	if (DigM==0) DigM=G4DigiManager::GetDMpointer()->FindDigitizerModule(SensitiveDetectorName) ;
	HC=new E391aHitsCollection(SensitiveDetectorName,collectionName[0]) ;
	for(G4int j=0 ; j<xHitMax ; j++) HitID[j]=-1 ;
	MaxTrack=0 ;
	}

G4bool E391aCCo4SensitiveDetector::ProcessHits(G4Step* Stp,G4TouchableHistory* RO) {
	G4TouchableHistory* TV=(G4TouchableHistory*)(Stp->GetPreStepPoint()->GetTouchable()) ;
// Scintillator bar copyNo ...
//++	ScNo=Stp->GetPreStepPoint()->GetPhysicalVolume()->GetCopyNo() ;
// Collar calorimeter module copyNo ...
//++	CNo=TV->GetVolume(1)->GetCopyNo() ;
// Always one Hit ... FOR INNEFICIENCY ESTIMATE
//	CNo=0 ; 
// Particle type ...
	ParID=E391aPType::PType(Stp->GetTrack()->GetDefinition()) ;
// Track seq num ...
	TrackID=Stp->GetTrack()->GetTrackID() ;
	if (TrackID>MaxTrack) MaxTrack=TrackID ;
	HitFill(0,Stp) ;
	return true ;
	}

void	E391aCCo4SensitiveDetector::HitFill(G4int module,G4Step *S) {
	G4StepPoint*	SP=S->GetPreStepPoint() ;
	if (module>=xHitMax) return ;
	if (HitID[module]==-1) {
		HitID[module]=HC->insert(new E391aHit(CCBaseID*10000+module,ParID,S))-1 ;
		(*HC)[HitID[module]]->AddEDep(SP->GetKineticEnergy()) ;
		}
	else (*HC)[HitID[module]]->AddEDep(S->GetTotalEnergyDeposit()) ;
// For parametrized version only ...
	S->GetTrack()->SetTrackStatus(fKillTrackAndSecondaries) ;
	}

void	E391aCCo4SensitiveDetector::EndOfEvent(G4HCofThisEvent* HCE) {
// Hist Fill ...
	G4int j ;
	G4double Ek=0 , DE=0 ;
	G4double xx=0 , yy=0 ;
	G4bool	CCo4Hit=false ;
	for(j=0 ; j<xHitMax ; j++) {
		if (HitID[j]!=-1) {
			CCo4Hit=true ;
			xx=(*HC)[HitID[j]]->X() ;
			yy=(*HC)[HitID[j]]->Y() ;
			DE+=(*HC)[HitID[j]]->EDep() ;
			Ek+=(*HC)[HitID[j]]->KineticEnergy() ;
			}
		}
	if (CCo4Hit) {
		CCo4Spect->Fill(Ek/GeV) ;
		CCo4Effo->Fill(Ek) ;
		CCo4Dep->Fill(DE) ;
		if (DE > ESensitivityThreshold) CCo4Eff->Fill(Ek) ;
		else	     CCo4HitXYu->Fill(xx,yy) ;
		CCo4HitXY->Fill(xx,yy) ;
		CCo4NTrk->Fill(float(MaxTrack)) ;
		}
// Hist End ...
static	G4int	HCID=-1 ; // Collection idetifier ...
	if (HCID<0) HCID=G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]) ;
	HCE->AddHitsCollection(HCID,HC) ;
//oo	if (DigM) DigM->Digitize() ;
	}

// EOF
