// File:	E391aCCo5-SD.cc
// Date:	16-Oct-2001 G.Macharashvili Juelich
// Updated:	

#include "E391aCCo5-SD.hh"

E391aCCo5SensitiveDetector::E391aCCo5SensitiveDetector(G4String DetName)
	:G4VSensitiveDetector(DetName) {
//oo		DigM=0 ;
		collectionName.insert(DetName) ;
		HitID=new G4int[xHitMax] ;

	CCo5NTrk=new TH1F("CCo5NTrk","Secondary tracks number in CCo5",100,0,100) ;
		CCo5NTrk->SetXTitle("Track number") ;
		CCo5NTrk->SetYTitle("Number of events") ;
	CCo5Spect=new TH1F("CCo5Spect","Initial Energy in CCo5",100,0,2.5) ;
		CCo5Spect->SetXTitle("energy,  GeV") ;
		CCo5Spect->SetYTitle("Number of events") ;
	CCo5Dep=new TH1F("CCo5Dep","Deposited Energy in CCo5",100,0,50) ;
		CCo5Dep->SetXTitle("Deposited energy,  MeV") ;
		CCo5Dep->SetYTitle("Number of events") ;
	CCo5HitXY=new TH2F("CCo5HitXY","Hit point distribution in CCo5",70,-175,+175,70,-175,+175) ;
		CCo5HitXY->SetXTitle("X coordinate,  mm") ;
		CCo5HitXY->SetXTitle("Y coordinate,  mm") ;
	CCo5HitXYu=new TH2F("CCo5HitXYu","Hit point distribution in CCo5 Undetected",70,-175,+175,70,-175,+175) ;
		CCo5HitXYu->SetXTitle("X coordinate,  mm") ;
		CCo5HitXYu->SetXTitle("Y coordinate,  mm") ;
	CCo5Effo=new TH1F("CCo5Effo","Initial Energy in CCo5",25,0,500) ;
		CCo5Effo->SetXTitle("Energy,  MeV") ;
		CCo5Effo->SetYTitle("Number of events") ;
	CCo5Eff=new TH1F("CCo5Eff","Detection inefficiency in CCo5 (#DeltaE<1 MeV)",25,0,500) ;
		CCo5Eff->SetXTitle("Energy,  MeV") ;
		CCo5Eff->SetYTitle("Number of events") ;
		}

E391aCCo5SensitiveDetector::~E391aCCo5SensitiveDetector() { delete [] HitID ; }

void E391aCCo5SensitiveDetector::Initialize(G4HCofThisEvent* HCE) {
//oo	if (DigM==0) DigM=G4DigiManager::GetDMpointer()->FindDigitizerModule(SensitiveDetectorName) ;
	HC=new E391aHitsCollection(SensitiveDetectorName,collectionName[0]) ;
	for(G4int j=0 ; j<xHitMax ; j++) HitID[j]=-1 ;
	MaxTrack=0 ;
	}

G4bool E391aCCo5SensitiveDetector::ProcessHits(G4Step* Stp,G4TouchableHistory* RO) {
	G4TouchableHistory* TV=(G4TouchableHistory*)(Stp->GetPreStepPoint()->GetTouchable()) ;
// Scintillator bar copyNo ...
//++	ScNo=Stp->GetPreStepPoint()->GetPhysicalVolume()->GetCopyNo() ;
// Collar calorimeter module copyNo ...
//++	CNo=TV->GetVolume(1)->GetCopyNo() ;
// Always one Hit ... FOR INNEFICIENCY ESTIMATE
//	CNo=0 ; 
// Particle type ...
	ParID=E391aPType::PType(Stp->GetTrack()->GetDefinition()) ;
// Track seq num ...
	TrackID=Stp->GetTrack()->GetTrackID() ;
	if (TrackID>MaxTrack) MaxTrack=TrackID ;
	HitFill(0,Stp) ;
	return true ;
	}

void	E391aCCo5SensitiveDetector::HitFill(G4int module,G4Step *S) {
	G4StepPoint*	SP=S->GetPreStepPoint() ;
	if (module>=xHitMax) return ;
	if (HitID[module]==-1) HitID[module]=HC->insert(new E391aHit(CCBaseID*10000+module,ParID,S))-1 ;
	else (*HC)[HitID[module]]->AddEDep(S->GetTotalEnergyDeposit()) ;
// For parametrized version only ...
	S->GetTrack()->SetTrackStatus(fKillTrackAndSecondaries) ;
	}

void	E391aCCo5SensitiveDetector::EndOfEvent(G4HCofThisEvent* HCE) {
// Hist Fill ...
	G4int j ;
	G4double Ek=0 , DE=0 ;
	G4double xx=0 , yy=0 ;
	G4bool	CCo5Hit=false ;
	for(j=0 ; j<xHitMax ; j++) {
		if (HitID[j]!=-1) {
			CCo5Hit=true ;
			xx=(*HC)[HitID[j]]->X() ;
			yy=(*HC)[HitID[j]]->Y() ;
			DE+=(*HC)[HitID[j]]->EDep() ;
			Ek+=(*HC)[HitID[j]]->KineticEnergy() ;
			}
		}
	if (CCo5Hit) {
		CCo5Spect->Fill(Ek/GeV) ;
		CCo5Effo->Fill(Ek) ;
		CCo5Dep->Fill(DE) ;
		if (DE > ESensitivityThreshold) CCo5Eff->Fill(Ek) ;
		else	     CCo5HitXYu->Fill(xx,yy) ;
		CCo5HitXY->Fill(xx,yy) ;
		CCo5NTrk->Fill(float(MaxTrack)) ;
		}
// Hist End ...
static	G4int	HCID=-1 ; // Collection idetifier ...
	if (HCID<0) HCID=G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]) ;
	HCE->AddHitsCollection(HCID,HC) ;
//oo	if (DigM) DigM->Digitize() ;
	}

// EOF
