// File:	E391aCalorimeterSD.cc
// Date:	03-May-2001 G.Macharashvili Dubna
// Updated:	13-Jun-2001 G.Macharashvili Dubna
// Updated:	02-Nov-2001 G.Macharashvili Dubna

#include "E391aCalorimeterSD.hh"

E391aCalorimeterSD::E391aCalorimeterSD(
	G4String DetName,
	E391aDetectorConstruction* det,
	E391aCalorimeterConfig* cfg)
		:
		G4VSensitiveDetector(DetName),Config(cfg),xDet(det) {
		collectionName.insert(DetName) ;
		HitID=new G4int[xHitMax] ;
		}

E391aCalorimeterSD::~E391aCalorimeterSD() { delete [] HitID ; }

void E391aCalorimeterSD::Initialize(G4HCofThisEvent* HCE) {
	HC=new E391aHitsCollection(SensitiveDetectorName,collectionName[0]) ;
	for(G4int j=0 ; j<xHitMax ; j++) HitID[j]=-1 ;
	}

G4bool E391aCalorimeterSD::ProcessHits(G4Step* Stp,G4TouchableHistory* RO) {
//	G4cout<<"CalorimeterSD::ProcessHits() ... \n" ;
	G4int		id=Config->Base()+Stp->GetPreStepPoint()->GetPhysicalVolume()->GetCopyNo() ;
	ParID=E391aPType::PType(Stp->GetTrack()->GetDefinition()) ;
	TrackID=Stp->GetTrack()->GetTrackID() ;
	HitFill(id,Stp) ;
	return true ;
	}

void	E391aCalorimeterSD::HitFill(G4int id,G4Step *S) {
	G4StepPoint*	SP=S->GetPreStepPoint() ;
	G4double	E=(ParID<3) ? SP->GetKineticEnergy() : S->GetTotalEnergyDeposit() ;
//	G4ThreeVector	D=SP->GetMomentumDirection() ;
	G4int	ix=Config->XNo(id) ;
	G4int	iy=Config->YNo(id) ;
	if (!Config->Test(ix,iy)) return ;
	if (TrackID>=xHitMax) return ;
	if (HitID[TrackID]==-1) {
		E391aHit* Hit=new E391aHit(id,ParID,S) ;
		Hit->SetEDep(E) ;
		HitID[TrackID]=HC->insert(Hit)-1 ;
		}
	else {
		(*HC)[HitID[TrackID]]->AddEDep(E) ;
		(*HC)[HitID[TrackID]]->AddPath(S->GetStepLength()) ;
		}
	if (ParID<3) {
		S->GetTrack()->SetTrackStatus(fKillTrackAndSecondaries) ;
/*
		G4cout<<"***** TrkID: "<<TrackID ;
		G4cout<<"  StpID: "<<S->GetTrack()->GetCurrentStepNumber() ;
		G4cout<<"  ParID: "<<ParID ;
		G4cout<<"  SLen: "<<S->GetStepLength() ;
		G4cout<<"  DE: "<<E<<endl ;
*/
		}
	else {
//		G4String VName1=S->GetPreStepPoint()->GetPhysicalVolume()->GetName() ;
		G4String VName2=S->GetPostStepPoint()->GetPhysicalVolume()->GetName() ;
		if (VName2!="EC.Module") S->GetTrack()->SetTrackStatus(fKillTrackAndSecondaries) ;
/*
		G4cout<<"TrkID: "<<TrackID ;
		G4cout<<"  StpID: "<<S->GetTrack()->GetCurrentStepNumber() ;
		G4cout<<"  ParID: "<<ParID ;
		G4cout<<"  SLen: "<<S->GetStepLength() ;
		G4cout<<"  DE: "<<E ;
		G4cout<<"  Vol: "<<VName2<<endl ;
*/
		}
	}

void	E391aCalorimeterSD::EndOfEvent(G4HCofThisEvent* HCE) {
static	G4int	HCollectionID=-1 ;
	if (HCollectionID<0) HCollectionID=G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]) ;
	HCE->AddHitsCollection(HCollectionID,HC) ;
	}

// EOF
