
#include "E391aDetectorConstruction.hh"

/*
Detector List
=============

1.  BeamVeto1       CeF3
2.  BeamVeto2       CeF3
3.  SBarrel         Lead/Scint
4.  BBarrel         Lead/Scint
5.  BeamDown1       Lead/Scint
6.  BeamDown2       Lead/Scint
7.  Calorimeter     CsI
8.  Collar          Tungsten/Scint
9.  CVeto           Scintillator
10. BCatcher        ?


IMPORTANT:
Geometry data have been taken from "Mechanical Structure
of the Detector for the KoL->pio nu nu Experiment"
by T.Inagaki (Draft: March 8, 2001)

*/

E391aDetectorConstruction::E391aDetectorConstruction() {
	UL=0 ;
	}

E391aDetectorConstruction::~E391aDetectorConstruction() {
	if (UL!=0) delete UL ;
	delete EMConfig ;
	delete EMFilter ;
	}

G4VPhysicalVolume*	E391aDetectorConstruction::Construct() {
	E391aMaterials() ;
	E391aDefaultGeometry() ;
	return E391aConstruct() ;
	}

G4VPhysicalVolume*	E391aDetectorConstruction::E391aConstruct() {

const	G4bool	VD=true ;
const	G4bool	BBVis=false ;
const	G4bool	SBVis=false ;

const	G4Color	BeamRed(1,0.4,0) ;
const	G4Color	PMGrey(0.8,0.8,0.8) ;
const	G4Color	EColor(1,1,0.5) ;
const	G4Color	SColor(1,1,1) ;
const	G4Color	WColor(0,0,0.8) ;
const	G4Color	SteelColor(0.2,0.2,0.2) ;
const	G4Color	ScColor2(0.2,0.8,0.8) ;
const	G4Color	VetoColor(0.4,0.6,1.0) ;

// *******************************************

	DD=new DummyDetector("DD") ;
	G4SDManager::GetSDMpointer()->AddNewDetector(DD) ;

// *******************************************
// General Mother Volume (Exp.Hall E391a)
	SolWorld=new G4Box("World",3.0*m,3.0*m,15.0*m) ;
	LogWorld=new G4LogicalVolume(SolWorld,HVacuum,"World",0,0,0) ;
	PhyWorld=new G4PVPlacement(0,G4ThreeVector(0,0,0),"World",LogWorld,0,false,0) ;
/*
	LogWorld->SetUserLimits(new G4UserLimits("Gamma-V1",DBL_MAX,DBL_MAX,1000*ns,2.5*MeV,1*mm)) ;
	LogWorld->SetUserLimits(new G4UserLimits("Electron-V1",DBL_MAX,DBL_MAX,1000*ns,2.5*MeV,1*mm)) ;
	LogWorld->SetUserLimits(new G4UserLimits("Positron-V1",DBL_MAX,DBL_MAX,1000*ns,2.5*MeV,1*mm)) ;
	LogWorld->SetUserLimits(new G4UserLimits("Charged-V1",DBL_MAX,DBL_MAX,1000*ns,2.5*MeV,1*mm)) ;
	LogWorld->SetUserLimits(new G4UserLimits("Neutron-V1",DBL_MAX,DBL_MAX,1000*ns,2.5*MeV,1*mm)) ;
	LogWorld->SetUserLimits(new G4UserLimits("KLong-V1",DBL_MAX,DBL_MAX,1000*ns,10*MeV,1*mm)) ;
*/
	G4VisAttributes	*WorldVisAtt=new G4VisAttributes(false,G4Color(1,1,1,1)) ;
	LogWorld->SetVisAttributes(WorldVisAtt) ;
// *******************************************
// Beam 
/*
{
	G4double	zo=-10*m ;
	G4double	z1=-117*cm ;
	G4double	z2=+893*cm ;
	G4double	vx=2*mrad ;
	SolBeam=new G4Cons("Beam",0,(z1-zo)*sin(vx),0,(z2-zo)*sin(vx),(z2-z1)/2,0,360*deg) ;
	LogBeam=new G4LogicalVolume(SolBeam,Vacuum,"Beam",0,0,0) ;
	PhyBeam=new G4PVPlacement(0,G4ThreeVector(0,0,(z1+z2)/2),"Beam",LogBeam,PhyWorld,false,0) ;
	G4VisAttributes	*BeamVisAtt=new G4VisAttributes(true,BeamRed) ;
	BeamVisAtt->SetForceSolid(true) ;
	LogBeam->SetVisAttributes(BeamVisAtt) ;
}
*/
// *******************************************
// EM calorimeter
{
// Configuration ...
	EMConfig=new E391aCalorimeterConfig ;
// PM Housings ...
const	G4double	PMZ=215*mm ;
const	G4double	Ri=56/2*mm ;
const	G4double	Ro=60/2*mm ;
	SolPM=new G4Tubs("EC.PM",Ri,Ro,PMZ/2,0,360*deg) ;
	LogPM=new G4LogicalVolume(SolPM,Iron,"EC.PM",0,0,0) ;
	G4VisAttributes	*PMVisAtt=new G4VisAttributes(true,PMGrey) ;
	PMVisAtt->SetForceSolid(true) ;
	LogPM->SetVisAttributes(PMVisAtt) ;

// Crystal Modules ...
const	G4double	DelZ=300*mm ;
const	G4double	DelY=6.99*cm ;
const	G4double	DelX=6.99*cm ;
	G4double	Xc , Yc , Zc=6020*mm+DelZ/2 ;
	SolECModule=new G4Box("EC",DelX/2,DelY/2,DelZ/2) ;
	LogECModule=new G4LogicalVolume(SolECModule,CsI,"EC",0,0,0) ;
	for (int ix=0 ; ix<EMConfig->NumOfX() ; ix++) {
		for (int iy=0 ; iy<EMConfig->NumOfY() ; iy++) {
			if (!EMConfig->Test(ix,iy)) continue ;
			Xc=EMConfig->XCenter(ix) ;
			Yc=EMConfig->YCenter(iy) ;
PhyECModule[EMConfig->Index(ix,iy)]=new G4PVPlacement(0,G4ThreeVector(Xc,Yc,Zc),"EC",LogECModule,PhyWorld,false,iy*100+ix) ;
PhyPM[EMConfig->Index(ix,iy)]=new G4PVPlacement(0,G4ThreeVector(Xc,Yc,Zc+DelZ/2+PMZ/2),"EC.PM",LogPM,PhyWorld,false,0) ;
			}
		}
// Set <StepMax,TrackMax,TimeMax,EkineMin,RangeMin> limits for LogicalVolume
/*
	LogECModule->SetUserLimits(new G4UserLimits("Gamma-EC",DBL_MAX,DBL_MAX,1000*ns,2.5*MeV,1*mm)) ;
	LogECModule->SetUserLimits(new G4UserLimits("Electron-EC",DBL_MAX,DBL_MAX,1000*ns,2.5*MeV,1*mm)) ;
	LogECModule->SetUserLimits(new G4UserLimits("Positron-EC",DBL_MAX,DBL_MAX,1000*ns,2.5*MeV,1*mm)) ;
	LogECModule->SetUserLimits(new G4UserLimits("Charged-EC",DBL_MAX,DBL_MAX,1000*ns,2.5*MeV,1*mm)) ;
*/
// EM Shower Model ...
//	new EMShowerModel("EM Shower Model",LogECModule) ;
// EM Calorimeter Detector:
	DetEC=new E391aCalorimeterSD("EC",this,EMConfig) ;
//--	DetEC=(E391aCalorimeterSD*) new DummyDetector("EC") ;
	G4SDManager::GetSDMpointer()->AddNewDetector(DetEC) ;
	LogECModule->SetSensitiveDetector(DetEC) ;
// Hit Digitization:
	DigitizerEC=new E391aCalorimeterDigitizer("EC",EMConfig) ;
	G4DigiManager::GetDMpointer()->AddNewModule(DigitizerEC) ;
// Reconstruction/Analysis:
	EMFilter=new E391aCalorimeterFilter(this,EMConfig) ;
// Vis Attributes
	G4VisAttributes	*ECVisAtt=new G4VisAttributes(true,EColor) ;
	ECVisAtt->SetForceSolid(true) ;
	LogECModule->SetVisAttributes(ECVisAtt) ;

}
// *******************************************
// Collar Calorimeter (Modular)
{
const	G4double	DelX=348/3*2*mm ;
const	G4double	DelY=348/3*1*mm ;
	G4double	DelZ=400*mm ;
const	G4double	FrontEndZ=5990*mm ;
const	G4double	Zc=FrontEndZ+DelZ/2 ;
 	G4double	Xc , Yc ;
static	const	G4double	RotAngZ[4]={0 , -90*deg , 180*deg , 90*deg} ;
	SolCC=new G4Box("CCo3",DelX/2,DelY/2,DelZ/2) ;
	LogCC=new G4LogicalVolume(SolCC,LVacuum,"CCo3",0,0,0) ;
	for (int k=0 ; k<4 ; k++) {
		RMCC[k]=new G4RotationMatrix() ;
		RMCC[k]->rotateZ(RotAngZ[k]) ;
		}
	PhyCC[0]=new G4PVPlacement(RMCC[0],G4ThreeVector( DelY/2, DelX/2,Zc),"CCo3",LogCC,PhyWorld,false,0) ;
	PhyCC[1]=new G4PVPlacement(RMCC[1],G4ThreeVector(-DelX/2, DelY/2,Zc),"CCo3",LogCC,PhyWorld,false,1) ;
	PhyCC[2]=new G4PVPlacement(RMCC[2],G4ThreeVector(-DelY/2,-DelX/2,Zc),"CCo3",LogCC,PhyWorld,false,2) ;
	PhyCC[3]=new G4PVPlacement(RMCC[3],G4ThreeVector( DelX/2,-DelY/2,Zc),"CCo3",LogCC,PhyWorld,false,3) ;
/*
	LogCC->SetUserLimits(new G4UserLimits("Gamma-CC",DBL_MAX,DBL_MAX,1000*ns,0.5*MeV,1*mm)) ;
	LogCC->SetUserLimits(new G4UserLimits("Electron-CC",DBL_MAX,DBL_MAX,1000*ns,0.5*MeV,1*mm)) ;
	LogCC->SetUserLimits(new G4UserLimits("Positron-CC",DBL_MAX,DBL_MAX,1000*ns,0.5*MeV,1*mm)) ;
	LogCC->SetUserLimits(new G4UserLimits("Neutron-CC",DBL_MAX,DBL_MAX,1000*ns,2.5*MeV,1*mm)) ;
*/
	E391aCC3SensitiveDetector *DetCCo3=new E391aCC3SensitiveDetector("CCo3") ;
	
	G4SDManager::GetSDMpointer()->AddNewDetector(DetCCo3) ;
	LogCC->SetSensitiveDetector(DetCCo3) ;

	DummyDigitizer *DigCCo3=new DummyDigitizer("CCo3",2) ;
	G4DigiManager::GetDMpointer()->AddNewModule(DigCCo3) ;

	G4VisAttributes	*CCVisAtt=new G4VisAttributes(VD,SColor) ;
	CCVisAtt->SetForceSolid(true) ;
	LogCC->SetVisAttributes(CCVisAtt) ;

// Layered structure of Collar calorimeter 1 Module
const	G4double	DW=1.0*mm ;
const	G4double	DS=5.0*mm ; // 3 or 5
const	G4double	SP=0.4*mm ; // Spacer ...
	DelZ-=1*mm ;
	SolW=new G4Box("CCo3.W",DelX/2,DW/2,DelZ/2) ;
	LogW=new G4LogicalVolume(SolW,Tungsten,"CCo3.W",0,0,0) ;
	SolS=new G4Box("CCo3.S",DelX/2,DS/2,DelZ/2) ;
	LogS=new G4LogicalVolume(SolS,Scintillator,"CCo3.S",0,0,0) ;
	G4int		Ns=int((DelY-DS)/(DS+DW+SP))+1 ;
	G4double	xo=DS/2-DelY/2 ;
	G4double	x1=(DelY-DS)/(Ns-1) ;
	G4double	yo=(x1+DS)/2-DelY/2 ;
	G4double	y1=x1 ;
	G4double	y=xo-x1 ;
G4cout.form("CCo3::Ns:%3d W:%.1f/S:%.1f mm.  Spacer:%.1f mm.\n",Ns,DW,DS,SP) ;
	for (int j=0 ; j<Ns ; j++) {
		y+=x1 ;
		PhyS[j]=new G4PVPlacement(0,G4ThreeVector(0,y,0),LogS,"CCo3.S",LogCC,false,j) ;
		}
	y=yo-y1 ;
	for (j=0 ; j<Ns-1 ; j++) {
		y+=y1 ;
		PhyW[j]=new G4PVPlacement(0,G4ThreeVector(0,y,0),LogW,"CCo3.W",LogCC,false,j) ;
		}
// ...
	LogS->SetSensitiveDetector(DetCCo3) ;
	LogW->SetSensitiveDetector(DetCCo3) ;
//
	G4VisAttributes	*WVA=new G4VisAttributes(true,WColor) ;
	WVA->SetForceSolid(true) ;
	LogW->SetVisAttributes(WVA) ;
	G4VisAttributes	*SVA=new G4VisAttributes(true,SColor) ;
	SVA->SetForceSolid(true) ;
	LogS->SetVisAttributes(SVA) ;

// Supporting Box
const	G4double	DelXx=112*mm ;
const	G4double	DelYx=  2*mm ;
const	G4double	DelZx=400*mm ;
const	G4double	Zx=5990*mm+DelZ/2 ;
	SolX=new G4Box("CCo3.Box",DelXx/2,DelYx/2,DelZx/2) ;
	LogX=new G4LogicalVolume(SolX,StainlessSteel,"CCo3.B",0,0,0) ;
	PhyX[0]=new G4PVPlacement(RMCC[3],G4ThreeVector(DelXx/2+DelYx/2,0,Zx),"CCo3.B",LogX,PhyWorld,false,0) ;
	PhyX[1]=new G4PVPlacement(RMCC[2],G4ThreeVector(0,-DelXx/2-DelYx/2,Zx),"CCo3.B",LogX,PhyWorld,false,1) ;
	PhyX[2]=new G4PVPlacement(RMCC[1],G4ThreeVector(-DelXx/2-DelYx/2,0,Zx),"CCo3.B",LogX,PhyWorld,false,2) ;
	PhyX[3]=new G4PVPlacement(RMCC[0],G4ThreeVector(0,DelXx/2+DelYx/2,Zx),"CCo3.B",LogX,PhyWorld,false,3) ;
/*
	LogX->SetUserLimits(new G4UserLimits("Any",DBL_MAX,DBL_MAX,1000*ns,1.5*MeV,2*mm)) ;
*/
	G4VisAttributes	*XVisAtt=new G4VisAttributes(true,SteelColor) ;
	XVisAtt->SetForceSolid(true) ;
	LogX->SetVisAttributes(XVisAtt) ;

}
// *******************************************
// Charged Veto
{
const	G4double	DelZ=10*mm ;
const	G4double	Zc=5620*mm+DelZ/2 ; // ?????
const 	G4double	Ri=50*mm ; // ***********
const	G4double	Ro=960*mm ; // ***********
const	G4double	Tmin= 0*deg+0.01*deg ; // Cracks !!!
const	G4double	Tmax=30*deg-0.01*deg ;
	SolCH=new G4Tubs("CH",Ri,Ro,DelZ/2,Tmin,Tmax) ;
	LogCH=new G4LogicalVolume(SolCH,Scintillator,"CH",0,0,0) ;
	G4double	a=15*deg ;
	for (int k=0 ; k<CHNumOfModules ; k++) {
		a+=30*deg ;
		RMCH[k]=new G4RotationMatrix() ;
		RMCH[k]->rotateZ(a) ;
		PhyCH[k]=new G4PVPlacement(RMCH[k],G4ThreeVector(0,0,Zc),"CH",LogCH,PhyWorld,false,k) ;
	}
/*
	LogCH->SetUserLimits(new G4UserLimits("Gamma-CH",DBL_MAX,DBL_MAX,1000*ns,0.01*MeV,1*mm)) ;
	LogCH->SetUserLimits(new G4UserLimits("Charged-CH",DBL_MAX,DBL_MAX,1000*ns,0.01*MeV,1*mm)) ;
*/
	DetCH=new E391aChargedVetoDetector("CH",this) ;
	G4SDManager::GetSDMpointer()->AddNewDetector(DetCH) ;
	LogCH->SetSensitiveDetector(DetCH) ;

	DigiCH=new E391aCHDigitizer("CH") ;
	G4DigiManager::GetDMpointer()->AddNewModule(DigiCH) ;

	G4VisAttributes	*CHVisAtt=new G4VisAttributes(true,ScColor2) ;
	CHVisAtt->SetForceSolid(true) ;
	LogCH->SetVisAttributes(CHVisAtt) ;


}
// *******************************************
// INTERNAL Charged Veto
{
const	G4double	Z1=5620*mm+10*mm ; // End of charged veto
const	G4double	Z2=5990*mm ; // Front of Collar
const	G4double	DelZ=Z2-Z1 ; // Cylinder Full Length
const 	G4double	Ri=60*mm ; // The same as for Charged veto
const	G4double	Ro=Ri+6*mm ; // Scintillator Thickness
	SolCHo=new G4Tubs("CHo",Ri,Ro,DelZ/2,0,360*deg) ;
	LogCHo=new G4LogicalVolume(SolCHo,Scintillator,"CHo",0,0,0) ;
	PhyCHo=new G4PVPlacement(0,G4ThreeVector(0,0,(Z1+Z2)/2),"CHo",LogCHo,PhyWorld,false,0) ;

//	LogCHo->SetUserLimits(new G4UserLimits("Gamma",DBL_MAX,DBL_MAX,1000*ns,1.5*MeV,1*mm)) ;

	G4VisAttributes	*CHVisAtt=new G4VisAttributes(true,ScColor2) ;
	CHVisAtt->SetForceSolid(true) ;
	LogCHo->SetVisAttributes(CHVisAtt) ;

//++	LogCHo->SetSensitiveDetector(DetCH) ;

}
// *******************************************
// SBarrel 16 Modules
{
const	G4double	DelZ=2750*mm ;
const	G4double	DelY=409.640*mm ;
const	G4double	DelX=294.993*mm ;
const	G4double	DelXv=125.315*mm ;
const	G4double	Omega=22.5*degree ;
const	G4double	Rin=315*mm ;

	SolSBModule=new G4Trap("SB",DelY/2,Omega/2,0,DelZ/2,DelXv/2,DelXv/2,0,DelZ/2,DelX/2,DelX/2,0) ;
	LogSBModule=new G4LogicalVolume(SolSBModule,SBMat,"SB",0,0,0) ;
	G4double	x , y , z=DelZ/2 ;
	G4double	a=0 ;
	for (int k=0 ; k<SBNumOfModules ; k++) {
		a+=Omega ;
		x=(Rin+DelY/2)*cos(a)+(DelX-DelXv)/4*sin(a) ;
		y=(Rin+DelY/2)*sin(a)-(DelX-DelXv)/4*cos(a) ;
		RM[k]=new G4RotationMatrix() ;
		RM[k]->rotateX(90*deg) ;
		RM[k]->rotateY(a-90*deg) ;
		PhySBModule[k]=new G4PVPlacement(RM[k],G4ThreeVector(x,y,z),"SB",LogSBModule,PhyWorld,false,k) ;
		}
/*
	LogSBModule->SetUserLimits(new G4UserLimits(DBL_MAX,DBL_MAX,1000*ns,1.*MeV,1*mm)) ;
*/
// Make Detector: ... nimushi ...
	DetSB=new E391aSBarrelSD("SB",this,SBNumOfModules) ;
	G4SDManager::GetSDMpointer()->AddNewDetector(DetSB) ;
	LogSBModule->SetSensitiveDetector(DetSB) ;
// visulaizaciis nimushi ...
	G4VisAttributes	*SBVisAtt=new G4VisAttributes(SBVis,VetoColor) ;
	SBVisAtt->SetForceSolid(true) ;
	LogSBModule->SetVisAttributes(SBVisAtt) ;

}
// *******************************************

// Big Barrel 32 Modules
{
const	G4double	DelZ=5500*mm ;
const	G4double	DelY=402.680*mm ;
const	G4double	DelX=276.096*mm ;
const	G4double	DelXv=195.998*mm ;
const	G4double	Omega=11.25*degree ;
const	G4double	Rin=995*mm ;

	SolBBModule=new G4Trap("BB",DelY/2,Omega/2,0,DelZ/2,DelXv/2,DelXv/2,0,DelZ/2,DelX/2,DelX/2,0) ;
	LogBBModule=new G4LogicalVolume(SolBBModule,BBMat,"BB",0,0,0) ;
	G4double	x , y , z=1223.7*mm+DelZ/2 ;
	G4double	a=0 ;
	for (int k=0 ; k<BBNumOfModules ; k++) {
		a+=Omega ;
		x=(Rin+DelY/2)*cos(a)-(DelX-DelXv)/4*sin(a) ;
		y=(Rin+DelY/2)*sin(a)+(DelX-DelXv)/4*cos(a) ;
		RMBB[k]=new G4RotationMatrix() ;
		RMBB[k]->rotateX(-90*deg) ;
		RMBB[k]->rotateY(-a-90*deg) ;
		PhyBBModule[k]=new G4PVPlacement(RMBB[k],G4ThreeVector(x,y,z),"BB",LogBBModule,PhyWorld,false,k) ;
		}
/*
	LogBBModule->SetUserLimits(new G4UserLimits(DBL_MAX,DBL_MAX,1000*ns,1.*MeV,1*mm)) ;
*/
// Make Detector
	DetBB=new E391aBBarrelSD("BB",this,BBNumOfModules) ;
	G4SDManager::GetSDMpointer()->AddNewDetector(DetBB) ;
	LogBBModule->SetSensitiveDetector(DetBB) ;
// VisAttributes
	G4VisAttributes	*BBVisAtt=new G4VisAttributes(BBVis,VetoColor) ;
	BBVisAtt->SetForceSolid(true) ;
	LogBBModule->SetVisAttributes(BBVisAtt) ;
}
// *******************************************
// Upstream Beam Veto 1
{
	G4double	z1=30*mm ;
	G4double	z2=z1+250*mm ;
	G4double	Ri=40*mm ; // ***********
	G4double	Ro=310*mm ;
	SolUB1=new G4Tubs("UBeamVeto1",Ri,Ro,(z2-z1)/2,0,360*deg) ;
	LogUB1=new G4LogicalVolume(SolUB1,Air,"UBeamVeto1",0,0,0) ;
	PhyUB1=new G4PVPlacement(0,G4ThreeVector(0,0,(z1+z2)/2),"UBeamVeto1",LogUB1,PhyWorld,false,0) ;
//	LogUB1->SetUserLimits(new G4UserLimits(DBL_MAX,DBL_MAX,1000*ns,1.*MeV,1*mm)) ;
	G4VisAttributes	*UB1VisAtt=new G4VisAttributes(true,G4Color(0.9,0.0,1.0)) ;
	UB1VisAtt->SetForceSolid(true) ;
	LogUB1->SetVisAttributes(UB1VisAtt) ;

	LogUB1->SetSensitiveDetector(DD) ;

}
// *******************************************
// Upstream Beam Veto 2
{
	G4double	z1=2500*mm ;
	G4double	z2=z1+250*mm ;
	G4double	Ri=40*mm ; // ***********
	G4double	Ro=310*mm ;
	SolUB2=new G4Tubs("UBeamVeto2",Ri,Ro,(z2-z1)/2,0,360*deg) ;
	LogUB2=new G4LogicalVolume(SolUB2,Air,"UBeamVeto2",0,0,0) ;
	PhyUB2=new G4PVPlacement(0,G4ThreeVector(0,0,(z1+z2)/2),"UBeamVeto2",LogUB2,PhyWorld,false,0) ;
//	LogUB2->SetUserLimits(new G4UserLimits(DBL_MAX,DBL_MAX,1000*ns,1.*MeV,1*mm)) ;
	G4VisAttributes	*UB2VisAtt=new G4VisAttributes(true,G4Color(0.9,0.0,1.0)) ;
	UB2VisAtt->SetForceSolid(true) ;
	LogUB2->SetVisAttributes(UB2VisAtt) ;

	LogUB2->SetSensitiveDetector(DD) ;

}
// *******************************************
// Downstream Beam Veto 1. == Collar Calorimeter CCo4

	E391aCCo4 *CCo4=new E391aCCo4() ;
	LogCCo4=CCo4->Construct(PhyWorld) ;


// *******************************************
// Downstream Beam Veto 2. == Collar Calorimeter CCo5

	E391aCCo5 *CCo5=new E391aCCo5() ;
	LogCCo5=CCo5->Construct(PhyWorld) ;

// *******************************************
// Beam Catcher
{
	G4double	z1=8930*mm ;
	G4double	z2=z1+200*mm ;
	G4double	Ro=150*mm ;
	SolBCatcher=new G4Tubs("BeamCatcher",0,Ro,(z2-z1)/2,0,360*deg) ;
	LogBCatcher=new G4LogicalVolume(SolBCatcher,CsI,"BeamCatcher",0,0,0) ;
	PhyBCatcher=new G4PVPlacement(0,G4ThreeVector(0,0,(z1+z2)/2),"BeamCatcher",LogBCatcher,PhyWorld,false,0) ;
//	LogBCatcher->SetUserLimits(new G4UserLimits(DBL_MAX,DBL_MAX,1000*ns,1.*MeV,1*mm)) ;
	G4VisAttributes	*BCatcherVisAtt=new G4VisAttributes(VD,G4Color(0.9,0.0,1.0)) ;
	BCatcherVisAtt->SetForceSolid(true) ;
	LogBCatcher->SetVisAttributes(BCatcherVisAtt) ;

	LogBCatcher->SetSensitiveDetector(DD) ;

}
// *******************************************

//	... G4UserLimits(Name,Stepmax,TrackMax,TimeMax,EkinMin,RangeMin) ;

	return PhyWorld ;

	}

void E391aDetectorConstruction::E391aDefaultGeometry() {}

void E391aDetectorConstruction::E391aMaterials() {
	G4String	name, symbol ;
	G4double	a, z, density ;
	G4double	pressure , temperature ;
	G4int		ncomponents, natoms ;
	G4double	abundance, fractionmass ;

H= new G4Element("Hydrogen",  "H",  1,    1.00794*g/mole) ;
C= new G4Element("Carbon",    "C",  6,   12.011*g/mole) ;
N= new G4Element("Nitrogen",  "N",  7,   14.007*g/mole) ;
O= new G4Element("Oxygen",    "O",  8,   15.999*g/mole) ;
Ar=new G4Element("Argon",    "Ar", 18,   39.948*g/mole) ;
F= new G4Element("Fluor",     "F",  9,   18.998*g/mole) ;
Cr=new G4Element("Chromium", "Cr", 24,   51.996*g/mole) ;
Mn=new G4Element("Manganese","Mn", 25,   54.938*g/mole) ;
Fe=new G4Element("Iron",     "Fe", 26,   55.845*g/mole) ;
Ni=new G4Element("Nickel",   "Ni", 28,   58.693*g/mole) ;
I =new G4Element("Iodine",    "I", 53,  126.904*g/mole) ;
Cs=new G4Element("Cesium",   "Cs", 55,  132.9054*g/mole) ;
Ce=new G4Element("Cerium",   "Ce", 58,  140.116*g/mole) ;
W =new G4Element("Tungsten",  "W", 74,  183.840*g/mole) ;
Pb=new G4Element("Lead",     "Pb", 82,  207.200*g/mole) ;

Al=new G4Material("Aluminium",  13,  26.98*g/mole,  2.700*g/cm3) ;
Cu=new G4Material("Copper",     29,  63.546*g/mole, 8.960*g/cm3) ;

Iron=new G4Material("Iron",7.870*g/cm3,1) ;
	Iron->AddElement(Fe,1) ;

Tungsten=new G4Material("Tungsten",19.250*g/cm3,1) ;
	Tungsten->AddElement(W,1) ;

Lead=new G4Material("Lead",11.35*g/cm3,1) ;
	Lead->AddElement(Pb,1) ;

CsI=new G4Material("CsI",4.51*g/cm3,2) ;
	CsI->AddElement(I, .5) ;
	CsI->AddElement(Cs,.5) ;
 
CeF3=new G4Material("CeF3",6.16*g/cm3,2) ;
	CeF3->AddElement(F, 3) ;
	CeF3->AddElement(Ce,1) ;
 
Air=new G4Material("Air",1.214*mg/cm3,4) ;
	Air->AddElement(N, 0.7494) ;
	Air->AddElement(O, 0.2369) ;
	Air->AddElement(Ar,0.0129) ;
	Air->AddElement(H, 0.0008) ;

Mylar=new G4Material("Mylar",1.4*g/cm3,3) ;
	Mylar->AddElement(C,5) ;
	Mylar->AddElement(H,4) ;
	Mylar->AddElement(O,2) ;

Scintillator=new G4Material("Scintillator",1.032*g/cm3,2) ;
	Scintillator->AddElement(H,1) ;
	Scintillator->AddElement(C,1) ;

StainlessSteel=new G4Material("StainlessSteel",8.02*g/cm3,5) ;
	StainlessSteel->AddElement(Fe, 0.6996) ;
	StainlessSteel->AddElement(C , 0.0004) ;
	StainlessSteel->AddElement(Mn, 0.0100) ;
	StainlessSteel->AddElement(Cr, 0.1900) ;
	StainlessSteel->AddElement(Ni, 0.1000) ;

//Vacuum=new G4Material("Vacuum",1.e-8*g/cm3,1,kStateGas,STP_Temperature,2.e-6*bar) ;
	G4double Pres=1.e-5*pascal ;
	G4double Rho=177.32*(mg/cm3)*(Pres/atmosphere)/(STP_Temperature/kelvin) ;
	HVacuum=new G4Material("HighVacuum",Rho,1,kStateGas,STP_Temperature,Pres) ;
		HVacuum->AddMaterial(Air,1) ;
	Pres=1.e-1*pascal ;
	Rho=177.32*(mg/cm3)*(Pres/atmosphere)/(STP_Temperature/kelvin) ;
	LVacuum=new G4Material("LowVacuum",Rho,1,kStateGas,STP_Temperature,Pres) ;
		LVacuum->AddMaterial(Air,1) ;

SBMat=CsI ;
BBMat=CsI ;

// Final Print ...
/*
	G4cout<<*G4Material::GetMaterialTable()<<endl ;
	const	G4MaterialTable	*MT=G4Material::GetMaterialTable() ;
	const	G4int	NumOfMT=G4Material::GetNumberOfMaterials() ;
	for (int m=0 ; m<NumOfMT ; m++) {
		G4cout.form("%12.12s",(*MT)[m]->GetName().data()) ;
		G4cout.form("  Dens:%12.3e g/cm3",(*MT)[m]->GetDensity()/(g/cm3)) ;
		G4cout.form("  Xrad:%10.2f cm",(*MT)[m]->GetRadlen()/cm) ;
//		G4cout.form("  Xnucl:%12.2f cm",(*MT)[m]->GetNuclearInterLength()/cm) ;
		G4cout.form("  Pres:%12.2e pa",(*MT)[m]->GetPressure()/pascal) ;
		G4cout.form("\n") ;
		}
*/
	}
	
// EOF
