// File:	E391aSBarrelSD.cc
// Date:	10-Apr-2001 G.Macharashvili Juelich

#include "E391aSBarrelSD.hh"

E391aSBarrelSD::E391aSBarrelSD(
	G4String DetName,
	E391aDetectorConstruction* det,
	G4int	NumOfModules)
		:
		G4VSensitiveDetector(DetName),xDet(det) {
		xNumOfModules=NumOfModules ;
		collectionName.insert(DetName) ;
		HitID=new G4int[xNumOfModules] ;
		}

E391aSBarrelSD::~E391aSBarrelSD() { delete [] HitID ; }

void E391aSBarrelSD::Initialize(G4HCofThisEvent* HCE) {
//	G4cout<<"E391aSBarrelSD::Initialize() ...\n" ;
	HC=new E391aHitsCollection(SensitiveDetectorName,collectionName[0]) ;
	for(G4int j=0 ; j<xNumOfModules ; j++) HitID[j]=-1 ;
	}

G4bool E391aSBarrelSD::ProcessHits(G4Step* Stp,G4TouchableHistory* RO) {
//	G4cout<<"E391aSBarrelSD::ProcessHits() ... \n" ;
	G4int		module=Stp->GetPreStepPoint()->GetPhysicalVolume()->GetCopyNo() ;
	if (module<0 || module>xNumOfModules-1) return false ;
	G4int		id=xBaseIdent*10000+module ;
	G4double	gt=Stp->GetPreStepPoint()->GetGlobalTime() ;
	G4double	De=Stp->GetTotalEnergyDeposit() ;
	G4double	Sl=Stp->GetStepLength() ;
	G4int		Pid=-1 ;
	if (De<=0) return false ;
// History check/ where we are
	G4TouchableHistory* TH=(G4TouchableHistory*)Stp->GetPreStepPoint()->GetTouchable() ;
	G4String	VN=TH->GetVolume()->GetName() ;
// Mandatory (standard) logic
	if (HitID[module]==-1) {
		E391aHit* Hit=new E391aHit(id,Pid,0,De,gt) ;
		HitID[module]=HC->insert(Hit)-1 ;
		}
	else {
		(*HC)[HitID[module]]->AddEDep(De) ;
		}
// Kill Track ...
	Stp->GetTrack()->SetTrackStatus(fKillTrackAndSecondaries) ;

	return true ;
	}

void	E391aSBarrelSD::EndOfEvent(G4HCofThisEvent* HCE) {
//	G4cout<<"E391aSBarrelSD::EndOfEvent() ...\n" ;
static	G4int	HCollectionID=-1 ;
	if (HCollectionID<0) {
		HCollectionID=G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]) ;
		}
	HCE->AddHitsCollection(HCollectionID,HC) ;
//	for (int k=0 ; k<xNumOfModules ; k++) {
//		if (HitID[k]!=-1) (*HC)[HitID[k]]->Print() ;
//		}
	}

void E391aSBarrelSD::clear() {} 

void E391aSBarrelSD::DrawAll() {} 

void E391aSBarrelSD::PrintAll() {} 

// EOF
