// File:	XParticle.hh
// Date:	07-Aug-2001 G.M. Juelich

#ifndef XParticle_hh
#define XParticle_hh 1

#include "E391aPType.hh"

class XParticle {

protected:

	short		ParticleID ;
	short		Reserved ;
	float		ThetaLab , Phi , EKin ;

public:

virtual	~XParticle() ;

	XParticle(G4ParticleDefinition*pdf,G4ThreeVector&mom) {
		ParticleID=E391aPType::PType(pdf) ;
		ThetaLab=mom.theta() ;
		Phi=mom.phi() ;
		double m=pdf->GetPDGMass() ;
		EKin=sqrt(mom.mag2()+m*m)-m ;
		}

	XParticle(G4int pID,G4ThreeVector&mom) {
		ParticleID=pID ;
		ThetaLab=mom.theta() ;
		Phi=mom.phi() ;
		double m=E391aPType::Definition(pID)->GetPDGMass() ;
		EKin=sqrt(mom.mag2()+m*m)-m ;
		}

	XParticle(G4int pID,G4double th,G4double fi,G4double ek) {
		ParticleID=pID ;
		ThetaLab=th ;
		Phi=fi ;
		EKin=ek ;
		}

	XParticle(G4DynamicParticle *x) {
		ParticleID=E391aPType::PType(x->GetDefinition()) ;
		ThetaLab=x->GetMomentumDirection().theta() ;
		Phi=x->GetMomentumDirection().phi() ;
		EKin=x->GetKineticEnergy() ;
		}

	XParticle(const XParticle &x) {
		ParticleID=x.ParticleID ;
		ThetaLab=x.ThetaLab ;
		Phi=x.Phi ;
		EKin=x.EKin ;
		}

const	XParticle& operator=(const G4DynamicParticle &x) {
		ParticleID=E391aPType::PType(x.GetDefinition()) ;
		ThetaLab=x.GetMomentumDirection().theta() ;
		Phi=x.GetMomentumDirection().phi() ;
		EKin=x.GetKineticEnergy() ;
		}

const	XParticle& operator=(const XParticle &x) {
		ParticleID=x.ParticleID ;
		ThetaLab=x.ThetaLab ;
		Phi=x.Phi ;
		EKin=x.EKin ;
		}

	G4ParticleDefinition*		Definition() {
		return E391aPType::Definition(ParticleID) ;
		}
	G4ThreeVector			Dir() {
		return G4ThreeVector(cos(ThetaLab)*sin(Phi),sin(ThetaLab)*sin(Phi),cos(ThetaLab)) ;
		}
	G4ThreeVector			Mom() {
		return Dir()*KineticEnergy()*sqrt(KineticEnergy()+2*Definition()->GetPDGMass()) ;
		}
	G4LorentzVector			Mom4() ;
	G4double			KineticEnergy() { return EKin ; }
	G4double			Mass() { return Definition()->GetPDGMass() ; }
	G4int				PType() { return ParticleID ; }
	G4DynamicParticle		DynamicParticle() {
		return G4DynamicParticle(Definition(),Mom()) ; 
		}

	void				SetParticle(G4ParticleDefinition*) ;
	void				SetParticle(G4int PType) { ParticleID=PType ; }
	void				SetMom(G4ThreeVector*) ;
	void				SetDir(G4ThreeVector*) ;

	} ;

#endif

// EOF
